/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */

#line 1 "chain/chain_parse.rl"

#include <string>
#include <list>

#line 37 "chain/chain_parse.rl"



#line 32 "chain/chain_parse.cpp"
static const char _flux_actions[] = {
	0, 1, 0, 1, 1, 1, 4, 1, 
	5, 2, 1, 2, 2, 1, 3
};

static const char _flux_key_offsets[] = {
	0, 0, 2, 4, 5, 6, 8, 11
};

static const char _flux_trans_keys[] = {
	42, 44, 42, 44, 42, 42, 42, 44, 
	0, 42, 44, 0, 42, 44, 0
};

static const char _flux_single_lengths[] = {
	0, 2, 2, 1, 1, 2, 3, 3
};

static const char _flux_range_lengths[] = {
	0, 0, 0, 0, 0, 0, 0, 0
};

static const char _flux_index_offsets[] = {
	0, 0, 3, 6, 8, 10, 13, 17
};

static const char _flux_trans_targs[] = {
	0, 0, 2, 3, 1, 2, 0, 4, 
	5, 4, 0, 0, 6, 7, 0, 5, 
	6, 7, 0, 5, 6, 0
};

static const char _flux_trans_actions[] = {
	0, 0, 1, 9, 3, 0, 0, 5, 
	7, 0, 0, 0, 1, 12, 0, 3, 
	0, 12, 0, 3, 0, 0
};

static const int flux_start = 1;
static const int flux_first_final = 7;
static const int flux_error = 0;

static const int flux_en_chain = 1;

#line 40 "chain/chain_parse.rl"

bool chain_parse(std::string const &text, std::list<std::string> &inputs, std::string &parser_name, std::list<std::string> &outputs)
{
  int cs;
  char const *p = text.c_str();
  char const *pe = text.c_str() + text.size() + 1;
  char const *mark;
  //char const *eof;
  std::list<std::string> current;

  
#line 89 "chain/chain_parse.cpp"
	{
	cs = flux_start;
	}
#line 51 "chain/chain_parse.rl"
  
#line 95 "chain/chain_parse.cpp"
	{
	int _klen;
	unsigned int _trans;
	const char *_acts;
	unsigned int _nacts;
	const char *_keys;

	if ( p == pe )
		goto _test_eof;
	if ( cs == 0 )
		goto _out;
_resume:
	_keys = _flux_trans_keys + _flux_key_offsets[cs];
	_trans = _flux_index_offsets[cs];

	_klen = _flux_single_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + _klen - 1;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + ((_upper-_lower) >> 1);
			if ( (*p) < *_mid )
				_upper = _mid - 1;
			else if ( (*p) > *_mid )
				_lower = _mid + 1;
			else {
				_trans += (_mid - _keys);
				goto _match;
			}
		}
		_keys += _klen;
		_trans += _klen;
	}

	_klen = _flux_range_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + (_klen<<1) - 2;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + (((_upper-_lower) >> 1) & ~1);
			if ( (*p) < _mid[0] )
				_upper = _mid - 2;
			else if ( (*p) > _mid[1] )
				_lower = _mid + 2;
			else {
				_trans += ((_mid - _keys)>>1);
				goto _match;
			}
		}
		_trans += _klen;
	}

_match:
	cs = _flux_trans_targs[_trans];

	if ( _flux_trans_actions[_trans] == 0 )
		goto _again;

	_acts = _flux_actions + _flux_trans_actions[_trans];
	_nacts = (unsigned int) *_acts++;
	while ( _nacts-- > 0 )
	{
		switch ( *_acts++ )
		{
	case 0:
#line 29 "chain/chain_parse.rl"
	{ mark=p; }
	break;
	case 1:
#line 29 "chain/chain_parse.rl"
	{ current.push_back(std::string(mark, p-mark)); }
	break;
	case 2:
#line 30 "chain/chain_parse.rl"
	{ inputs.insert(inputs.begin(), current.begin(), current.end()); current.clear(); }
	break;
	case 3:
#line 31 "chain/chain_parse.rl"
	{ outputs.insert(outputs.begin(), current.begin(), current.end()); current.clear(); }
	break;
	case 4:
#line 32 "chain/chain_parse.rl"
	{ mark=p; }
	break;
	case 5:
#line 32 "chain/chain_parse.rl"
	{ parser_name = std::string(mark, p-mark); }
	break;
#line 192 "chain/chain_parse.cpp"
		}
	}

_again:
	if ( cs == 0 )
		goto _out;
	if ( ++p != pe )
		goto _resume;
	_test_eof: {}
	_out: {}
	}
#line 52 "chain/chain_parse.rl"

  if ( (cs == flux_error) || (cs < flux_first_final) )
    return false;

  return true;
}

