/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_CHAINMANAGER_
#define _FLUX_CHAINMANAGER_

#include <string>
#include <list>
#include <map>

#include "chain.hpp"

namespace flux
{
  class ChainManager
  {
    public:
      template <typename T>
      struct RegInfo
      {
         typename T::Allocator allocator;
         std::string  description;

         RegInfo() 
         : allocator(0), description("Unknown") {}

         RegInfo(typename T::Allocator a, std::string const &d) 
         : allocator(a), description(d) {}
      };

      typedef std::map< io::IOChecker, RegInfo<io::Input> > InputTypesMap;
      typedef std::map< io::IOChecker, RegInfo<io::Output> > OutputTypesMap;
      typedef std::map< std::string, RegInfo<parse::Parser> > ParserNamesMap;

    private:
      typedef std::map< std::string, RefCounted<io::Input> > AllocatedInputMap;
      typedef std::map< std::string, Chain* > ChainsMap;

    private:
      InputTypesMap input_types_;
      OutputTypesMap output_types_;
      ParserNamesMap parser_names_;

      AllocatedInputMap inputs_;
      ChainsMap chains_;

    public:
      ChainManager();
      ~ChainManager();

      bool registerInput(io::IOChecker checker, RegInfo<io::Input> const &info);
      bool registerOutput(io::IOChecker checker, RegInfo<io::Output> const &info);
      bool registerParser(std::string const &name, RegInfo<parse::Parser> const &info);

      std::string createChain(std::list<std::string> const &input, std::string const &parser, std::list<std::string> const &output, std::string const &id = "");
      void cycle();

      inline InputTypesMap const &getInputs() { return input_types_; }
      inline OutputTypesMap const &getOutputs() { return output_types_; }
      inline ParserNamesMap const &getParsers() const { return parser_names_; }
  };
}

#endif

