/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */

#include "config.h"

#include "io.hpp"

#include <cstdlib>
#include <cerrno>

#if ENABLE_DEBUG
#include <iostream>
#endif

namespace flux
{
  namespace io
  {
    Input::Input(char const *uri) : uri_(uri), fd_(-1) {}
    Input::~Input()
    {
      #if ENABLE_DEBUG
        std::cerr << "DELETING INPUT: " << this << " -> " << uri_ << std::endl;
      #endif
    }

    Output::Output(char const *uri) : uri_(uri) {}
    Output::~Output()
    {
      #if ENABLE_DEBUG
        std::cerr << "DELETING OUTPUT: " << this << " -> " << uri_ << std::endl;
      #endif
    }

    bool is_a_port(std::string const &what)
    {
      if(what.empty())
        return false;

      errno = 0;
      int port = strtol(what.c_str(), (char**)NULL, 10);

      if ((!port) || (errno) || (port % 2) || (port >= 0xFFFF))
        return false;

      return true;
    }
  }
}

