/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_IO_INTERFACE_
#define _FLUX_IO_INTERFACE_

#include <string>

#include "../rtp/packet.hpp"

namespace flux
{
  namespace io
  {
    typedef bool(*IOChecker)(std::string const &def);
    bool is_a_port(std::string const &what);

    class Input
    {
      public:
        typedef class Input*(*Allocator)(std::string const &uri);

      protected:
        std::string uri_;
        rtp::Packet received_packet_;
        int fd_;

      public:
        explicit Input(char const *uri);
        virtual ~Input() = 0;

        virtual bool isAvailable() = 0;
        virtual bool recover() = 0;

        /* Each input must have an associated file descriptor that will be polled for data
         * when data is available the read call will be performed.
         */
        inline int getDescriptor() { return fd_; }

        /* feed() method is called by the main loop when there is no data associated with the Input descriptor
         * it should set the internal packet to a size of 0 and then do anything necessary to acquire more data
         * for the next loop iteration.
         */
		virtual void feed() = 0;

        virtual unsigned long read() = 0;
        virtual rtp::Packet &getPacket() = 0;
    };

    class Output
    {
      public:
        typedef class Output*(*Allocator)(std::string const &uri);

      protected:
        std::string uri_;

      public:
        explicit Output(char const *uri);
        virtual ~Output() = 0;

        virtual bool isAvailable() = 0;
        virtual bool recover() = 0;

        virtual unsigned long write(unsigned char *buffer, unsigned long buffer_size) = 0;
    };
  }
}

#endif

