/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_IO_FILE_
#define _FLUX_IO_FILE_

#include "io.hpp"
#include <iosfwd>

namespace flux
{
    namespace io
    {
        class FileInput : public Input
        {
            public:
                static bool Check(const std::string &uri);
                static Input *Alloc(const std::string &uri);

                explicit FileInput(char const *uri);
                virtual ~FileInput();

                virtual bool isAvailable();
                virtual bool recover();

                virtual void feed();
                virtual unsigned long read();
                virtual rtp::Packet &getPacket();
        };

        class FileOutput : public Output
        {
            private:
                std::ofstream *output_;

            public:
                static bool Check(const std::string &uri);
                static Output *Alloc(const std::string &uri);

                explicit FileOutput(char const *uri);
                virtual ~FileOutput();

                virtual bool isAvailable();
                virtual bool recover();

                virtual unsigned long write(unsigned char *buffer, unsigned long buffer_size);
        };
    }
}

#endif

