/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */


#ifndef FLUX_IO_MQ
#define FLUX_IO_MQ

#include "io.hpp"

#include <mqueue.h>
#include "utils/mq_metadata.hpp"

namespace flux
{
    namespace io
    {
        class MQOutput : public Output
        {
            public:
                static unsigned const QueueMessageSize = rtp::DEFAULT_MTU + MQMetaData::Size;
                static unsigned const MaxQueueMessages = 2048;

            private:
                MQMetaData meta_;
                mqd_t mqd_;
                int last_seq_;
                unsigned short seq_stray_;
                unsigned long prev_ts_;
                unsigned long delivery_ts_;

            public:
                static bool Check(const std::string &uri);
                static Output *Alloc(const std::string &uri);

                explicit MQOutput(char const *uri);
                virtual ~MQOutput();

                virtual bool isAvailable();
                virtual bool recover();

                virtual unsigned long write(unsigned char *buffer, unsigned long buffer_size);
        };
    }
}

#endif

