/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_IO_TCP_
#define _FLUX_IO_TCP_

#include "io.hpp"

namespace flux
{
  namespace io
  {
    class TCPInput : public Input
    {
      private:
        void *listen_socket_;
        void *transport_;
        bool ok_;
        unsigned long long int last_pkt_time_;

      public:
        static bool Check(const std::string &uri);
        static Input *Alloc(const std::string &uri);

        explicit TCPInput(char const *uri);
        virtual ~TCPInput();

        virtual bool isAvailable();
        virtual bool recover();

        virtual void feed();
        virtual unsigned long read();
        virtual rtp::Packet &getPacket();
    };

    class TCPOutput : public Output
    {
      private:
        void *transport_;
        bool ok_;

      private:
        static void parse_uri(std::string const &uri, std::string &host, std::string &port);

      public:
        static bool Check(const std::string &uri);
        static Output *Alloc(const std::string &uri);

        explicit TCPOutput(char const *uri);
        virtual ~TCPOutput();

        virtual bool isAvailable();
        virtual bool recover();

        virtual unsigned long write(unsigned char *buffer, unsigned long buffer_size);
    };
  }
}

#endif

