/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */

#include "config.h"

#include "parser.hpp"

#if ENABLE_DEBUG
#include <iostream>
#endif

namespace flux
{
  namespace parse
  {
    Parser::Parser() {}
    Parser::~Parser() {}

    CopyParser::CopyParser() : Parser() {}
    CopyParser::~CopyParser()
    {
        #if ENABLE_DEBUG
        std::cerr << "DELETING PARSER " << this << std::endl;
        #endif
    }

    Parser *CopyParser::Alloc(std::string const &opts)
    {
        return new CopyParser();
    }

    bool CopyParser::parse(rtp::Packet const &p)
    {
        if (!p.getBufferSize()) {
            p_ = 0;
            return false;
        }

        p_ = &p;
        return true;
    }

    bool CopyParser::hasMore() { return false; }

    unsigned char *CopyParser::getResult() { return p_ ? const_cast<rtp::Packet*>(p_)->getBuffer() : 0; }
    unsigned long CopyParser::getResultSize() { return p_ ? p_->getBufferSize() : 0; }
  }
}


#ifdef TEST
#include <iostream>
#include <cstring>
#include <sys/stat.h>

int main()
{
  flux::parse::CopyParser parser;
  flux::rtp::Packet p;
  struct stat file_info;

  stat("test.rtp", &file_info);
  FILE * f = fopen("test.rtp", "r");
  fread(p.getBuffer(), 1, file_info.st_size, f);
  fclose(f);

  p.parseBuffer(file_info.st_size);
  parser.parse(p);

  if (p.getBufferSize() != parser.getResultSize()) {
    std::cerr << "Copied packet size differs from original packet size " << p.getBufferSize() << " != " << parser.getResultSize() << std::endl;
    return 1;
  }

  if (memcmp(p.getBuffer(), parser.getResult(), parser.getResultSize()) != 0) {
    std::cerr << "Copied packet differs from original packet" << std::endl;
    return 2;
  }

  std::cout << "+ COPYPARSER TEST SUITE PASSED" << std::endl;
  return 0;
}

#endif

