/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_PARSER_TRANSCODE_
#define _FLUX_PARSER_TRANSCODE_

#include "parser.hpp"
#include "../utils/ffmpeg.hpp"

#include <list>

namespace flux
{
  namespace rtp { class Depacketizer; class Packetizer; }

  namespace parse
  {
    class TranscodeParser : public Parser
    {
      private:
        static int const MAX_BUFFER_SIZE = 32768;

        rtp::Depacketizer *depacker_;
        bool repack_;
        rtp::Packetizer *repacker_;
        FFmpeg ffmpeg_;

        unsigned char tmp_buffer_[MAX_BUFFER_SIZE];
        unsigned long tmp_buffer_size_;
        std::list<rtp::Packet*> result_buffers_;

        unsigned int output_width_;
        unsigned int output_height_;
        unsigned long ssrc_;

      private:
        bool transcode(void *, void *);

      public:
        static Parser *Alloc(std::string const &opts);
        static Parser *PackAlloc(std::string const &opts);

        TranscodeParser(bool repack, unsigned int ow, unsigned int oh);
        ~TranscodeParser();

        bool parse(rtp::Packet const &p);
        unsigned char *getResult();
        unsigned long getResultSize();
        bool hasMore();
    };
  }
}

#endif
