/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */

#include "config.h"

#include "unpack.hpp"
#include "../rtp/unpack/depacketizer.hpp"
#include "../rtp/unpack/factory.hpp"

#include <iostream>
#include <cstring>

namespace flux
{
  namespace parse
  {
    UnpackParser::UnpackParser()
    : Parser(), depacker_(0), buffer_(0)
    {}

    UnpackParser::~UnpackParser()
    {
      delete depacker_;
      delete[] buffer_;
    }

    Parser *UnpackParser::Alloc(std::string const &opts)
    {
      return new UnpackParser();
    }

    bool UnpackParser::parse(rtp::Packet const &p)
    {
        buffer_size_ = 0;

        if (!p.getBufferSize())
            return false;

        if (!depacker_)
            depacker_ = rtp::DepacketizerFactory::Create(p.getPayloadType());

        if (p.getPayloadType() != depacker_->getPayloadType()) {
            std::cerr << "Payload Type changed! " << depacker_->getPayloadType() << " expected, got " << p.getPayloadType() << std::endl;
            return false;
        }

        rtp::Depacketizer::FrameBuffer fb;
        rtp::Depacketizer::ConfigBuffer cb;
        rtp::Depacketizer::ParseResult result = depacker_->parse(p, &fb, &cb);

        if (result == rtp::Depacketizer::Ok) {
            if (buffer_)
                delete[] buffer_;

            buffer_size_ = fb.len;
            buffer_ = new unsigned char[fb.len];
            memcpy(buffer_, fb.data, fb.len);

            return true;
        }

        return false;
    }

    unsigned char *UnpackParser::getResult() { return buffer_; }
    unsigned long UnpackParser::getResultSize() { return buffer_size_; }
    bool UnpackParser::hasMore() { return false; }
  }
}

