/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */

#include "config.h"

#include <iostream>

#include "factory.hpp"

#include "packetizer.hpp"
#include "mpegvideo.hpp"

namespace flux
{
  namespace rtp
  {
    PacketizerFactory::PacketizerFactory() {}

    Packetizer *PacketizerFactory::Create(unsigned long ssrc, int payload_type, std::string const &name)
    {
        if (!name.empty()) {
            if (name == "MPV")
                payload_type = 32;
            else {
                std::cerr << "libRTP Packetizer Factory, unsupported payload name " << name << std::endl;
                return 0;
            }
        }

        switch (payload_type) {
            case 32:
                return new MpegVideoPacketizer(ssrc);
            default:
                std::cerr << "libRTP Packetizer Factory, unsupported payload type " << payload_type << std::endl;
                return 0;
        }
    }
  }
}
