/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_RTP_PACKET_
#define _FLUX_RTP_PACKET_

namespace flux
{
  namespace rtp
  {
    unsigned const MAX_CSRC_COUNT = 15;
    unsigned const MAX_PAYLOAD_SIZE = 9000;
    unsigned const DEFAULT_MTU = 1500;

    class Packet
    {
      private:
        static unsigned long const SsrcBase_ = 0xFE118;
        static unsigned long SsrcLast_;
        static unsigned long TimestampBase_;
      
      public:
        static unsigned long GenerateUniqueSsrc();
        static inline unsigned long GenerateSsrc() { return SsrcBase_; }
        static unsigned int TimestampNow();
        static inline unsigned long &CommonTimestamp() { return TimestampBase_ ? TimestampBase_ : (TimestampBase_ = TimestampNow()); }
        
      private: 
        unsigned short version;
        bool padding;
        bool extension;
        bool marker;

        unsigned long ssrc;
        unsigned long csrc[MAX_CSRC_COUNT];
        unsigned short csrc_count;

        unsigned short payload_type;
        unsigned short seq;
        unsigned long time;
        
        unsigned char data[MAX_PAYLOAD_SIZE];
        unsigned long data_size;

        unsigned char *payload_;
        unsigned long payload_size_;

      public:
        Packet();
        ~Packet();
        Packet &operator=(Packet const &other);

        inline unsigned short getVersion() { return version; }
        inline Packet &setVersion(unsigned short v) { version = v; return *this; }

        inline bool hasPadding() const { return padding; }
        inline Packet &setPadding(bool v) { padding = v; return *this; }

        inline bool hasExtension() const { return extension; }
        inline Packet &setExtension(bool v) { extension = v; return *this; }

        inline bool hasMarker() const { return marker; }
        inline Packet &setMarker(bool v) { marker = v; return *this; }

        inline unsigned long getSSRC() const { return ssrc; }
        inline Packet &setSSRC(unsigned long v) { ssrc = v; return *this; }

        inline unsigned short getCSRCCount() const { return csrc_count; }
        inline Packet &setCSRCCount(unsigned short v) { csrc_count = v < 16 ? v : 15; return *this; }

        unsigned long getCSRC(unsigned short index) const;
        Packet &setCSRC(unsigned short index, unsigned long v);
        Packet &addCSRC(unsigned long v);

        inline Packet &setPayloadType(unsigned short v) { payload_type = v; return *this; }
        inline unsigned short getPayloadType() const { return payload_type; }

        inline unsigned short getSeq() const { return seq; }
        inline Packet &setSeq(unsigned short v) { seq = v; return *this; }

        inline unsigned long getTimestamp() const { return time; }
        inline Packet &setTimestamp(unsigned long t) { time = t; return *this; }
        
        inline Packet &setBufferSize(unsigned long v) { data_size = v; return *this; }

        inline unsigned char *getBuffer() { return data; }    
        Packet &parseBuffer(unsigned short size);
        Packet &regenBuffer();

        inline unsigned char *payload() { return payload_; }
        inline unsigned char const *payload() const { return payload_; }
        inline unsigned long getPayloadSize() const { return payload_size_; }
        inline Packet &setPayloadSize(unsigned long ps) { payload_size_ = ps; return *this; }
        
        inline unsigned long getBufferSize() const { return data_size; }
        inline unsigned long getHeaderSize() const { return (12 + (csrc_count*4)); }
    };
  }
}

#endif

