/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */


#ifndef _FLUX_DEPACKETIZER_H_
#define _FLUX_DEPACKETIZER_H_

#include <string>

namespace flux
{
  namespace rtp
  {
    class Packet;

    class Depacketizer
    {
      public:
        enum ParseResult { Ok = 0,
                           NeedNextFragment = 1,
                           NotImplemented = -1, PacketEmpty = -2, MalformedPacket = -3,
                           LostFirstFragment = -4, OutOfMemory = -5 };

        struct ConfigBuffer
        {
            unsigned int len;
            unsigned char *data;
        };

        struct FrameBuffer
        {
            long len;
            unsigned int timestamp;
            double time_sec;
            int fps;
            unsigned char pt;
            unsigned char *data;
        };

      private:
        unsigned payload_type_;
        std::string name_;

      public:
        Depacketizer(unsigned payload_type, std::string const &name);
        virtual ~Depacketizer() = 0;

        inline const std::string &getName() const { return name_; }
        inline Depacketizer &setName(std::string const &name) { name_ = name; return *this; }

        inline unsigned getPayloadType() const { return payload_type_; }
        inline Depacketizer &setPayloadType(unsigned payload_type) { payload_type_ = payload_type; return *this; }

        virtual Depacketizer::ParseResult parse(Packet const &packet, Depacketizer::FrameBuffer *destination_frame_buffer, Depacketizer::ConfigBuffer *destination_config_buffer) = 0;
    };
  }
}

#endif
