/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_FFMPEG_H_
#define _FLUX_FFMPEG_H_

//Forward declaring ffmpeg things
typedef struct AVCodecContext AVCodecContext;
typedef struct SwsContext SwsContext;
typedef struct AVFrame AVFrame;
typedef struct AVCodec AVCodec;

namespace flux
{
    class FFmpeg
    {
        public:
            enum CodecId {
                CODEC_ID_NONE = 0,
                CODEC_ID_MPEG1VIDEO = 1,
                CODEC_ID_MPEG2VIDEO = 2
            };

        private:
            static bool init_done_;
            int root;

            AVCodecContext *decode_context_;
            AVFrame *input_frame_;

            AVCodecContext *encode_context_;
            AVFrame *output_frame_;
            SwsContext *resize_context_;

        protected:
            static int Init();

        public:
            FFmpeg();
            ~FFmpeg();

            int initEncodeContext(FFmpeg::CodecId codec_id, unsigned int ow, unsigned int oh);
            unsigned long encode(AVFrame *input, unsigned char *output, unsigned int output_size);

            int initDecodeContext(FFmpeg::CodecId codec_id);
            AVFrame *decode(unsigned char *input, unsigned int input_size);

            int initScaleContext(unsigned int width, unsigned int height);
            AVFrame *scaleFrame(AVFrame *orig_frame);

            static AVFrame *MakeBlackFrame(unsigned int width, unsigned int height);
            static void FreeFrame(AVFrame *f);
    };
}

#endif

