/* *
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 *
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * */

#include <cstring>

#include <sys/time.h>

#include "rtp/packet.hpp"
#include "mq_metadata.hpp"

namespace flux
{
    MQMetaData::StartTimesMap MQMetaData::StartTimes_;

    static double ev_time()
    {
        struct timeval tv;
        gettimeofday (&tv, 0);
        return tv.tv_sec + tv.tv_usec * 1e-6;
    }

    std::pair<double, unsigned int> &MQMetaData::GetStartTime(void *id, unsigned int cur_ts)
    {
        if (StartTimes_.find(id) == StartTimes_.end())
            StartTimes_[id] = std::make_pair(ev_time(), cur_ts);

        return StartTimes_[id];
    }

    MQMetaData::MQMetaData()
    {

    }

    MQMetaData::~MQMetaData()
    {

    }

    void MQMetaData::Rebase(void *id)
    {
        MQMetaData::StartTimesMap::iterator mq_data = StartTimes_.find(id);
        if (mq_data != StartTimes_.end())
            StartTimes_.erase(mq_data);
    }

    void MQMetaData::rebuild(void *id, unsigned int cur_ts)
    {
        std::pair<double, unsigned int> &data = GetStartTime(id, cur_ts);

        memcpy(buffer_, (char*)&data.first, sizeof(double));
        memcpy(buffer_+sizeof(double), (char*)&cur_ts, sizeof(unsigned int));
        memcpy(buffer_+sizeof(double)+sizeof(unsigned int), (char*)&data.second, sizeof(unsigned int));
    }
}


