/* * 
 * This file is part of Flux
 *
 * Copyright (C) 2009 by LScube team <team@lscube.org>
 * See AUTHORS for more details
 * 
 * flux is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with flux; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 * */


#ifndef _FLUX_REF_
#define _FLUX_REF_

#ifdef DEBUG
#include <iostream>
#endif

namespace flux
{
  template <typename T>
  class RefCounted
  {
    private:
      unsigned int *count_;
      T *who_;

    public:
      explicit RefCounted(T *who) : count_(new unsigned int(1)), who_(who) { }
      RefCounted(RefCounted<T> const &orig) : count_(orig.count_), who_(orig.who_)
      {
        if (count_)
          ++(*count_);
      }
      RefCounted() : count_(new unsigned int(1)), who_(0) {}

      RefCounted<T> &operator=(RefCounted<T> const &orig)
      {
        if ( !(--(*count_)) ) {
          delete who_;
          delete count_;
        }

        count_ = orig.count_;
        ++(*count_);
        who_ = orig.who_;

        return *this;
      }

      RefCounted<T> &operator=(T *p)
      {
        if ( !(--(*count_)) ) {
          delete who_;
          delete count_;
        }

        count_ = new unsigned int(1);
        who_ = p;

        return *this;
      }

      ~RefCounted()
      {
        if ( !(--(*count_)) ) {
          #ifdef DEBUG
          if (who_)
            std::cerr << "DELETING REFPTR: " << who_ << std::endl;
          #endif

          delete who_;
          delete count_;
        }
      }

      T &operator*() { return *who_; }
      T *operator->() { return who_; }
      T *ptr() { return who_; }
      bool operator!() { return !who_; }
  };
}

#endif
